"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generate = void 0;
const path_1 = require("path");
const mem_fs_1 = require("mem-fs");
const mem_fs_editor_1 = require("mem-fs-editor");
const cloneDeep_1 = __importDefault(require("lodash/cloneDeep"));
const project_access_1 = require("@sap-ux/project-access");
const config_1 = require("./config");
const file_1 = require("./file");
const constants_1 = require("./constants");
/**
 * Writes the template to the memfs editor instance.
 *
 * @param basePath - the base path
 * @param abapDeployConfig - the abap deploy config
 * @param options - the deploy config options
 * @param fs - the memfs editor instance
 * @returns the updated memfs editor instance
 */
function generate(basePath, abapDeployConfig, options, fs) {
    var _a, _b;
    return __awaiter(this, void 0, void 0, function* () {
        const abapConfig = (0, cloneDeep_1.default)(abapDeployConfig);
        if (!fs) {
            fs = (0, mem_fs_editor_1.create)((0, mem_fs_1.create)());
        }
        // base config
        const baseConfigFile = (_a = options === null || options === void 0 ? void 0 : options.baseFile) !== null && _a !== void 0 ? _a : project_access_1.FileName.Ui5Yaml;
        const baseConfig = yield (0, project_access_1.readUi5Yaml)(basePath, baseConfigFile, fs);
        const isLib = baseConfig.getType() === 'library';
        const baseConfigPath = (0, path_1.join)(basePath, baseConfigFile);
        (0, config_1.updateBaseConfig)(isLib, baseConfigPath, baseConfig, fs);
        // deploy config
        const deployConfigFile = (_b = options === null || options === void 0 ? void 0 : options.deployFile) !== null && _b !== void 0 ? _b : project_access_1.FileName.UI5DeployYaml;
        const deployFilePath = (0, path_1.join)(basePath, deployConfigFile);
        const deployConfig = yield (0, config_1.getDeployConfig)(abapConfig, baseConfig);
        fs.write(deployFilePath, deployConfig.toString());
        // package.json
        // deploy script
        const deployScript = `${constants_1.BUILD_SCRIPT} && ${constants_1.DEPLOY_SCRIPT} --config ${deployConfigFile} && ${constants_1.RIMRAF} archive.zip`;
        yield (0, project_access_1.updatePackageScript)(basePath, 'deploy', deployScript, fs);
        // undeploy script
        const undeployScript = `${constants_1.BUILD_SCRIPT} && ${constants_1.UNDEPLOY_SCRIPT} --config ${deployConfigFile}`;
        yield (0, project_access_1.updatePackageScript)(basePath, 'undeploy', undeployScript, fs);
        // test mode script
        const deployTestModeScript = `${constants_1.BUILD_SCRIPT} && ${constants_1.DEPLOY_SCRIPT} --config ${deployConfigFile} --testMode true`;
        yield (0, project_access_1.updatePackageScript)(basePath, 'deploy-test', deployTestModeScript, fs);
        // dependencies
        yield (0, project_access_1.addPackageDevDependency)(basePath, constants_1.RIMRAF, constants_1.RIMRAF_VERSION, fs);
        if (isLib) {
            yield (0, project_access_1.addPackageDevDependency)(basePath, constants_1.UI5_TASK_FLATTEN_LIB, constants_1.UI5_TASK_FLATTEN_LIB_VERSION, fs);
            (0, file_1.addUi5Dependency)(fs, basePath, constants_1.UI5_TASK_FLATTEN_LIB);
        }
        // ui5 repo files
        yield (0, file_1.writeUi5RepositoryFiles)(fs, basePath);
        return fs;
    });
}
exports.generate = generate;
//# sourceMappingURL=index.js.map